@@ -46,10 +46,11 @@ G_DEFINE_TYPE_EXTENDED (HalClient, hal_client, G_TYPE_OBJECT, 0, G_IMPLEMENT_INT
 
 typedef struct _HalClientPrivate HalClientPrivate;
 struct _HalClientPrivate {
-  DBusGConnection *system_bus_connection;
+  DBusGConnection *dbus_connection;
+  DBusGProxy *dbus_proxy;
   LibHalContext *hal_ctx;
 
-  GPtrArray *pending_devices;
+  gboolean online;
   GHashTable *udi_name_table;
 
   gboolean disposed;
@@ -57,63 +58,16 @@ struct _HalClientPrivate {
 
 #define HAL_CLIENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), HAL_CLIENT_TYPE, HalClientPrivate))
 
+#define DBUS_SERVICE "org.freedesktop.DBus"
+#define DBUS_IFACE   "org.freedesktop.DBus"
+#define DBUS_PATH    "/org/freedesktop/DBus"
+
+#define HAL_SERVICE   "org.freedesktop.Hal"
+#define HAL_MGR_PATH  "/org/freedesktop/Hal/Manager"
+#define HAL_MGR_IFACE "org.freedesktop.Hal.Manager"
 
 /* methods */
 
-static gboolean
-hal_device_get_rapi_connection(HalClient *self, WmDevice *device)
-{
-  RapiConnection *rapi_conn;
-  HRESULT hr;
-  gchar *name = NULL;
-  gchar *device_name;
-
-  if (!self) {
-    g_warning("%s: Invalid object passed", G_STRFUNC);
-    return FALSE;
-  }
-
-  HalClientPrivate *priv = HAL_CLIENT_GET_PRIVATE (self);
-
-  if (priv->disposed) {
-    g_warning("%s: Disposed object passed", G_STRFUNC);
-    return FALSE;
-  }
-
-  g_object_get(device, "name", &name, NULL);
-
-  rapi_conn = rapi_connection_from_name(name);
-  g_free(name);
-  if (!rapi_conn) {
-    g_critical("%s: Failed to obtain rapi connection to %s", G_STRFUNC, name);
-    goto error_exit;;
-  }
-
-  rapi_connection_select(rapi_conn);
-  CeRapiInit();
-
-  hr = CeRapiInit();
-  if (FAILED(hr)) {
-    g_critical("%s: Failed to initialise rapi connection to %s: %d: %s", G_STRFUNC, name, hr, synce_strerror(hr));
-    goto error_exit;;
-  }
-
-  device_name = get_device_name_via_rapi();
-  if (!(device_name)) {
-    CeRapiUninit();
-    goto error_exit;
-  }
-
-  g_object_set(device, "rapi-conn", rapi_conn, NULL);
-  g_object_set(device, "device-name", device_name, NULL);
-  g_free(device_name);
-
-  return TRUE;
-error_exit:
-  rapi_connection_destroy(rapi_conn);
-  return FALSE;
-}
-
 static void 
 password_status_changed_cb(LibHalContext *ctx,
 			  const char *udi,
@@ -139,10 +93,8 @@ password_status_changed_cb(LibHalContext *ctx,
   if (strcmp(key, "pda.pocketpc.password") != 0) 
     return;
 
-  gchar *pdaname = NULL, *tmpname;
-  gint i;
+  gchar *pdaname = NULL;
   DBusError dbus_error;
-  WmDevice *device = NULL;
   gchar *pw_status;
 
   dbus_error_init(&dbus_error);
@@ -153,25 +105,6 @@ password_status_changed_cb(LibHalContext *ctx,
     goto error_exit;;
   }
 
-  i = 0;
-  while (i < priv->pending_devices->len) {
-    device = g_ptr_array_index(priv->pending_devices, i);
-    g_object_get(device, "name", &tmpname, NULL);
-
-    if (g_ascii_strcasecmp(tmpname, pdaname) == 0) {
-      g_free(tmpname);
-      break;
-    }
-    device = NULL;
-    g_free(tmpname);
-    i++;
-  }
-
-  if (!device) {
-    g_warning("%s: Received password flags changed for unfound device: %s", G_STRFUNC, udi);
-    goto error_exit;;
-  }
-
   pw_status = libhal_device_get_property_string(priv->hal_ctx, udi, "pda.pocketpc.password", &dbus_error);
   if (dbus_error_is_set(&dbus_error)) {
     g_critical("%s: error getting property from device %s: %s: %s", G_STRFUNC, udi, dbus_error.name, dbus_error.message);
@@ -188,20 +121,15 @@ password_status_changed_cb(LibHalContext *ctx,
     goto exit;
   }
 
-  if ( (strcmp(pw_status, "unset") == 0) || (strcmp(pw_status, "unlocked")) ) {
-    g_ptr_array_remove_index(priv->pending_devices, i);
+  if ( (strcmp(pw_status, "unset") == 0) || (strcmp(pw_status, "unlocked") == 0) ) {
 
-    if (!libhal_device_remove_property_watch(priv->hal_ctx, udi, &dbus_error))
-      g_critical("%s: Failed to remove property watch for device %s: %s: %s", G_STRFUNC, udi, dbus_error.name, dbus_error.message);
+          if (!libhal_device_remove_property_watch(priv->hal_ctx, udi, &dbus_error)) {
+                  g_warning("%s: Failed to remove property watch for device %s: %s: %s", G_STRFUNC, udi, dbus_error.name, dbus_error.message);
+                  if (dbus_error_is_set(&dbus_error))
+                          dbus_error_free(&dbus_error);
+          }
 
-    /* get rapi connection */
-    if (!(hal_device_get_rapi_connection(self, device))) {
-      g_object_unref(device);
-      g_hash_table_remove(priv->udi_name_table, udi);
-      goto error_exit;
-    }
-
-    g_signal_emit(self, DCCM_CLIENT_GET_INTERFACE (self)->signals[DEVICE_CONNECTED], 0, pdaname, (gpointer)device);
+          g_signal_emit(self, DCCM_CLIENT_GET_INTERFACE (self)->signals[DEVICE_UNLOCKED], 0, pdaname);
   }
   goto exit;
 
@@ -309,6 +237,9 @@ hal_add_device(HalClient *self, const char *udi)
   transport = "hal";
 
   device = g_object_new(WM_DEVICE_TYPE,
+			"object-name", udi,
+                        "dccm-type", "hal",
+                        "connection-status", DEVICE_STATUS_UNKNOWN,
 			"name", name,
 			"os-major", os_major,
 			"os-minor", os_minor,
@@ -338,21 +269,23 @@ hal_add_device(HalClient *self, const char *udi)
       g_critical("%s: Failed to set property watch for device %s: %s: %s", G_STRFUNC, udi, dbus_error.name, dbus_error.message);
       goto error_exit;
     }
-    g_ptr_array_add(priv->pending_devices, device);
+
     g_hash_table_insert(priv->udi_name_table, g_strdup(udi), g_strdup(name));
 
-    if ((strcmp(pw_status, "provide") == 0))
-      g_signal_emit (self, DCCM_CLIENT_GET_INTERFACE (self)->signals[PASSWORD_REQUIRED], 0, name);
-    if ((strcmp(pw_status, "provide-on-device") == 0))
-      g_signal_emit (self, DCCM_CLIENT_GET_INTERFACE (self)->signals[PASSWORD_REQUIRED_ON_DEVICE], 0, name);
+    if ((strcmp(pw_status, "provide") == 0)) {
+            g_object_set(device, "connection-status", DEVICE_STATUS_PASSWORD_REQUIRED, NULL);
+            g_signal_emit(self, DCCM_CLIENT_GET_INTERFACE (self)->signals[DEVICE_CONNECTED], 0, name, (gpointer)device);
+            g_signal_emit(self, DCCM_CLIENT_GET_INTERFACE (self)->signals[PASSWORD_REQUIRED], 0, name);
+    }
+    if ((strcmp(pw_status, "provide-on-device") == 0)) {
+            g_object_set(device, "connection-status", DEVICE_STATUS_PASSWORD_REQUIRED_ON_DEVICE, NULL);
+            g_signal_emit(self, DCCM_CLIENT_GET_INTERFACE (self)->signals[DEVICE_CONNECTED], 0, name, (gpointer)device);
+            g_signal_emit (self, DCCM_CLIENT_GET_INTERFACE (self)->signals[PASSWORD_REQUIRED_ON_DEVICE], 0, name);
+    }
     goto exit;
   }
 
-  /* get rapi connection */
-  if (!(hal_device_get_rapi_connection(self, device))) {
-    goto error_exit;
-  }
-
+  g_object_set(device, "connection-status", DEVICE_STATUS_CONNECTED, NULL);
   g_hash_table_insert(priv->udi_name_table, g_strdup(udi), g_strdup(name));
   g_signal_emit(self, DCCM_CLIENT_GET_INTERFACE (self)->signals[DEVICE_CONNECTED], 0, name, (gpointer)device);
 
@@ -400,6 +333,7 @@ hal_device_disconnected_cb(LibHalContext *ctx, const char *udi)
   }
 
   gchar *pdaname = NULL;
+  DBusError dbus_error;
 
   pdaname = (gchar*) g_hash_table_lookup(priv->udi_name_table, udi);
 
@@ -408,92 +342,29 @@ hal_device_disconnected_cb(LibHalContext *ctx, const char *udi)
 
   g_debug("%s: Received device disconnected from hal: %s", G_STRFUNC, udi);
 
-  gint i;
-  gchar *tmp;
-  WmDevice *device;
+  dbus_error_init(&dbus_error);
 
-  i = 0;
-
-  while (i < priv->pending_devices->len) {
-    device = g_ptr_array_index(priv->pending_devices, i);
-    g_object_get(device, "name", &tmp, NULL);
-
-    if (g_ascii_strcasecmp(pdaname, tmp) == 0) {
-      libhal_device_remove_property_watch(priv->hal_ctx, udi, NULL);
-      g_ptr_array_remove_index(priv->pending_devices, i);
-      g_object_unref(device);
-      g_free(tmp);
-      goto exit;
-    }
-
-    g_free(tmp);
-    i++;
+  if (!(libhal_device_remove_property_watch(priv->hal_ctx, udi, &dbus_error))) {
+          g_warning("%s: Error removing watch on device: %s: %s", G_STRFUNC, dbus_error.name, dbus_error.message);
+          if (dbus_error_is_set(&dbus_error))
+                  dbus_error_free(&dbus_error);
   }
 
   g_signal_emit (self, DCCM_CLIENT_GET_INTERFACE (self)->signals[DEVICE_DISCONNECTED], 0, pdaname);
 
-exit:
   g_hash_table_remove(priv->udi_name_table, udi);
 
   return;
 }
 
-static void
-clear_pending_devices(gpointer data, gpointer user_data)
-{
-  g_object_unref(WM_DEVICE(data));
-  return;
-}
-
-gboolean
-hal_client_uninit_comms_impl(HalClient *self)
-{
-  if (!self) {
-    g_warning("%s: Invalid object passed", G_STRFUNC);
-    return FALSE;
-  }
-
-  HalClientPrivate *priv = HAL_CLIENT_GET_PRIVATE (self);
-
-  if (priv->disposed) {
-    g_warning("%s: Disposed object passed", G_STRFUNC);
-    return FALSE;
-  }
-  if (!priv->system_bus_connection) {
-    g_warning("%s: Uninitialised object passed", G_STRFUNC);
-    return TRUE;
-  }
-
-  DBusError dbus_error;
-  dbus_error_init(&dbus_error);
-
-  if (!libhal_ctx_shutdown(priv->hal_ctx, &dbus_error)) {
-    g_critical("%s: Failed to shutdown hal context: %s: %s", G_STRFUNC, dbus_error.name, dbus_error.message);
-    dbus_error_free(&dbus_error);
-  }
-
-  libhal_ctx_free(priv->hal_ctx);
-  dbus_g_connection_unref(priv->system_bus_connection);
-  priv->system_bus_connection = NULL;
-
-  g_ptr_array_foreach(priv->pending_devices, clear_pending_devices, NULL);
-  g_ptr_array_free(priv->pending_devices, TRUE);
-
-  g_hash_table_destroy(priv->udi_name_table);
-
-  return TRUE;
-}
-
 void
-hal_client_provide_password_impl(HalClient *self, gchar *pdaname, gchar *password)
+hal_client_provide_password_impl(HalClient *self, const gchar *pdaname, const gchar *password)
 {
   GError *error = NULL;
   gboolean password_accepted = FALSE, result;
-  gint i, num_devices;
+  gint num_devices;
   DBusGProxy *proxy = NULL;
   DBusError dbus_error;
-  WmDevice *device = NULL;
-  gchar *name;
   gchar **device_list;
 
   if (!self) {
@@ -507,32 +378,13 @@ hal_client_provide_password_impl(HalClient *self, gchar *pdaname, gchar *passwor
     g_warning("%s: Disposed object passed", G_STRFUNC);
     return;
   }
-  if (!priv->system_bus_connection) {
+  if (!priv->dbus_connection) {
     g_warning("%s: Uninitialised object passed", G_STRFUNC);
     return;
   }
 
   dbus_error_init(&dbus_error);
 
-  i = 0;
-  while (i < priv->pending_devices->len) {
-    device = g_ptr_array_index(priv->pending_devices, i);
-    g_object_get(device, "name", &name, NULL);
-
-    if (g_ascii_strcasecmp(pdaname, name) == 0) {
-      g_free(name);
-      break;
-    }
-    g_free(name);
-    device = NULL;
-    i++;
-  }
-
-  if (!device) {
-    g_warning("%s: Password provided for unfound device: %s", G_STRFUNC, pdaname);
-    return;
-  }
-
   device_list = libhal_manager_find_device_string_match(priv->hal_ctx,
                                                         "pda.pocketpc.name",
                                                         pdaname,
@@ -548,12 +400,12 @@ hal_client_provide_password_impl(HalClient *self, gchar *pdaname, gchar *passwor
     return;
   }
 
-  proxy = dbus_g_proxy_new_for_name(priv->system_bus_connection,
+  proxy = dbus_g_proxy_new_for_name(priv->dbus_connection,
 				    "org.freedesktop.Hal",
 				    device_list[0],
 				    "org.freedesktop.Hal.Device.Synce");
   if (proxy == NULL) {
-    g_critical("%s: Failed to get proxy for device '%s'", G_STRFUNC, device_list[i]);
+    g_critical("%s: Failed to get proxy for device '%s'", G_STRFUNC, device_list[0]);
     goto error_exit;
   }
 
@@ -576,16 +428,7 @@ hal_client_provide_password_impl(HalClient *self, gchar *pdaname, gchar *passwor
     goto exit;
   }
 
-  g_ptr_array_remove_index(priv->pending_devices, i);
-
-  /* get rapi connection */
-  if (!(hal_device_get_rapi_connection(self, device))) {
-      g_object_unref(device);
-      goto error_exit;
-  }
-
-  g_signal_emit(self, DCCM_CLIENT_GET_INTERFACE (self)->signals[DEVICE_CONNECTED], 0, pdaname, (gpointer)device);
-
+  g_debug("%s: Password accepted for %s", G_STRFUNC, pdaname);
   goto exit;
 error_exit:
   if (dbus_error_is_set(&dbus_error))
@@ -602,7 +445,7 @@ exit:
 }
 
 gboolean
-hal_client_request_disconnect_impl(HalClient *self, gchar *pdaname)
+hal_client_request_disconnect_impl(HalClient *self, const gchar *pdaname)
 {
   gboolean result = FALSE;
 
@@ -625,38 +468,53 @@ hal_client_request_disconnect_impl(HalClient *self, gchar *pdaname)
 }
 
 
-gboolean
-hal_client_init_comms_impl(HalClient *self)
+static void
+hal_disconnect(HalClient *self)
+{
+  if (!self) {
+    g_warning("%s: Invalid object passed", G_STRFUNC);
+    return;
+  }
+  HalClientPrivate *priv = HAL_CLIENT_GET_PRIVATE (self);
+
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return;
+  }
+
+  DBusError dbus_error;
+  dbus_error_init(&dbus_error);
+
+  if (!libhal_ctx_shutdown(priv->hal_ctx, &dbus_error)) {
+    g_critical("%s: Failed to shutdown hal context: %s: %s", G_STRFUNC, dbus_error.name, dbus_error.message);
+    dbus_error_free(&dbus_error);
+  }
+
+  libhal_ctx_free(priv->hal_ctx);
+  priv->hal_ctx = NULL;
+
+  g_hash_table_remove_all(priv->udi_name_table);
+}
+
+static void
+hal_connect(HalClient *self)
 {
-  GError *error = NULL;
   DBusError dbus_error;
-  gboolean result = FALSE;
   gchar **dev_list = NULL;
   gint i, num_devices;
   gchar *udi = NULL;
 
   if (!self) {
     g_warning("%s: Invalid object passed", G_STRFUNC);
-    goto exit;
+    return;
   }
   HalClientPrivate *priv = HAL_CLIENT_GET_PRIVATE (self);
 
-  if (priv->disposed) {
-    g_warning("%s: Disposed object passed", G_STRFUNC);
-    goto exit;
-  }
-  if (priv->system_bus_connection) {
-    g_warning("%s: Initialised object passed", G_STRFUNC);
-    goto exit;
-  }
-
   dbus_error_init(&dbus_error);
 
-  priv->system_bus_connection = dbus_g_bus_get (DBUS_BUS_SYSTEM,
-					  &error);
-  if (priv->system_bus_connection == NULL) {
-    g_critical("%s: Failed to open connection to bus: %s", G_STRFUNC, error->message);
-    goto error_exit;
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return;
   }
 
   if (!(priv->hal_ctx = libhal_ctx_new())) {
@@ -664,7 +522,7 @@ hal_client_init_comms_impl(HalClient *self)
     goto error_exit;
   }
 
-  if (!libhal_ctx_set_dbus_connection(priv->hal_ctx, dbus_g_connection_get_connection(priv->system_bus_connection))) {
+  if (!libhal_ctx_set_dbus_connection(priv->hal_ctx, dbus_g_connection_get_connection(priv->dbus_connection))) {
     g_critical("%s: Failed to set DBus connection for hal context", G_STRFUNC);
     goto error_exit;
   }
@@ -713,24 +571,157 @@ hal_client_init_comms_impl(HalClient *self)
   }
   libhal_free_string_array(dev_list);
 
-  result = TRUE;
-  goto exit;
+  return;
 
 error_exit:
-  if (error)
-    g_error_free (error);
+  priv->online = FALSE;
   if (dbus_error_is_set(&dbus_error))
     dbus_error_free(&dbus_error);
   if (priv->hal_ctx) {
     libhal_ctx_shutdown(priv->hal_ctx, NULL);
     libhal_ctx_free(priv->hal_ctx);
   }
-  if (priv->system_bus_connection) {
-    dbus_g_connection_unref(priv->system_bus_connection);
-    priv->system_bus_connection = NULL;
+  return;
+}
+
+static void
+hal_status_changed_cb(DBusGProxy *proxy,
+                      gchar *name,
+                      gchar *old_owner,
+                      gchar *new_owner,
+                      gpointer user_data)
+{
+        HalClient *self = HAL_CLIENT(user_data);
+        if (!self) {
+                g_warning("%s: Invalid object passed", G_STRFUNC);
+                return;
+        }
+        HalClientPrivate *priv = HAL_CLIENT_GET_PRIVATE (self);
+
+        if (priv->disposed) {
+                g_warning("%s: Disposed object passed", G_STRFUNC);
+                return;
+        }
+
+        if (strcmp(name, HAL_SERVICE) != 0)
+                return;
+
+        /* If this parameter is empty, odccm just came online */
+
+        if (strcmp(old_owner, "") == 0) {
+                priv->online = TRUE;
+                g_debug("%s: hal came online", G_STRFUNC);
+                hal_connect(self);
+
+                g_signal_emit (self, DCCM_CLIENT_GET_INTERFACE (self)->signals[SERVICE_STARTING], 0);
+                return;
+        }
+
+        /* If this parameter is empty, odccm just went offline */
+
+        if (strcmp(new_owner, "") == 0) {
+                priv->online = FALSE;
+                g_debug("%s: hal went offline", G_STRFUNC);
+                hal_disconnect(self);
+
+                g_signal_emit (self, DCCM_CLIENT_GET_INTERFACE (self)->signals[SERVICE_STOPPING], 0);
+                return;
+        }
+}
+
+gboolean
+hal_client_uninit_comms_impl(HalClient *self)
+{
+  if (!self) {
+    g_warning("%s: Invalid object passed", G_STRFUNC);
+    return FALSE;
   }
-exit:
-  return result;
+
+  HalClientPrivate *priv = HAL_CLIENT_GET_PRIVATE (self);
+
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return FALSE;
+  }
+  if (!priv->dbus_connection) {
+    g_warning("%s: Uninitialised object passed", G_STRFUNC);
+    return TRUE;
+  }
+
+  if (priv->hal_ctx)
+          hal_disconnect(self);
+
+  dbus_g_proxy_disconnect_signal (priv->dbus_proxy, "NameOwnerChanged",
+                                  G_CALLBACK(hal_status_changed_cb), NULL);
+
+  g_object_unref(priv->dbus_proxy);
+  priv->dbus_proxy = NULL;
+
+  dbus_g_connection_unref(priv->dbus_connection);
+  priv->dbus_connection = NULL;
+
+  return TRUE;
+}
+
+gboolean
+hal_client_init_comms_impl(HalClient *self)
+{
+  GError *error = NULL;
+  gboolean result = FALSE;
+  gboolean has_owner = FALSE;
+
+  if (!self) {
+    g_warning("%s: Invalid object passed", G_STRFUNC);
+    return FALSE;
+  }
+  HalClientPrivate *priv = HAL_CLIENT_GET_PRIVATE (self);
+
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return FALSE;
+  }
+  if (priv->dbus_connection) {
+    g_warning("%s: Initialised object passed", G_STRFUNC);
+    return FALSE;
+  }
+
+  priv->dbus_connection = dbus_g_bus_get (DBUS_BUS_SYSTEM,
+					  &error);
+  if (priv->dbus_connection == NULL) {
+    g_critical("%s: Failed to open connection to bus: %s", G_STRFUNC, error->message);
+    g_error_free (error);
+    return result;
+  }
+
+  priv->dbus_proxy = dbus_g_proxy_new_for_name (priv->dbus_connection,
+                                                DBUS_SERVICE,
+                                                DBUS_PATH,
+                                                DBUS_IFACE);
+
+  dbus_g_proxy_add_signal (priv->dbus_proxy, "NameOwnerChanged",
+			   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
+
+  dbus_g_proxy_connect_signal (priv->dbus_proxy, "NameOwnerChanged",
+			       G_CALLBACK(hal_status_changed_cb), self, NULL);
+
+  if (!(dbus_g_proxy_call(priv->dbus_proxy, "NameHasOwner",
+			  &error,
+                          G_TYPE_STRING, HAL_SERVICE,
+                          G_TYPE_INVALID,
+			  G_TYPE_BOOLEAN, &has_owner,
+			  G_TYPE_INVALID))) {
+          g_critical("%s: Error checking owner of %s: %s", G_STRFUNC, HAL_SERVICE, error->message);
+          g_error_free(error);
+          return TRUE;
+  }
+
+  if (has_owner) {
+          priv->online = TRUE;
+          hal_connect(self);
+  } else
+          priv->online = FALSE;
+
+  return TRUE;
 }
 
 
@@ -741,7 +732,11 @@ hal_client_init(HalClient *self)
 {
   HalClientPrivate *priv = HAL_CLIENT_GET_PRIVATE (self);
 
-  priv->pending_devices = g_ptr_array_new();
+  priv->dbus_connection = NULL;
+  priv->dbus_proxy = NULL;
+  priv->hal_ctx = NULL;
+
+  priv->online = FALSE;
   priv->udi_name_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 }
 
@@ -754,9 +749,11 @@ hal_client_dispose (GObject *obj)
   if (priv->disposed) {
     return;
   }
-  if (priv->system_bus_connection)
+  if (priv->dbus_connection)
     dccm_client_uninit_comms(DCCM_CLIENT(self));
 
+  g_hash_table_destroy(priv->udi_name_table);
+
   priv->disposed = TRUE;
 
   /* unref other objects */
@@ -796,6 +793,6 @@ dccm_client_interface_init (gpointer g_iface, gpointer iface_data)
 
   iface->dccm_client_init_comms = (gboolean (*) (DccmClient *self)) hal_client_init_comms_impl;
   iface->dccm_client_uninit_comms = (gboolean (*) (DccmClient *self)) hal_client_uninit_comms_impl;
-  iface->dccm_client_provide_password = (void (*) (DccmClient *self, gchar *pdaname, gchar *password)) hal_client_provide_password_impl;
-  iface->dccm_client_request_disconnect = (gboolean (*) (DccmClient *self, gchar *pdaname)) hal_client_request_disconnect_impl;
+  iface->dccm_client_provide_password = (void (*) (DccmClient *self, const gchar *pdaname, const gchar *password)) hal_client_provide_password_impl;
+  iface->dccm_client_request_disconnect = (gboolean (*) (DccmClient *self, const gchar *pdaname)) hal_client_request_disconnect_impl;
 }
