@@ -35,7 +35,6 @@
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <gnome.h>
-#include <glade/glade.h>
 #include <errno.h>
 
 #include "synce-trayicon.h"
@@ -121,9 +120,10 @@ main (gint argc, gchar **argv)
 			GNOME_PROGRAM_STANDARD_PROPERTIES,
 			NULL);
 
- 	synce_log_set_level(log_level);
+        g_set_application_name(_("SynCE Tray Icon"));
 
-	glade_gnome_init ();
+
+ 	synce_log_set_level(log_level);
 
 	if (in_background)
 	{
