@@ -29,13 +29,12 @@ IN THE SOFTWARE.
 #include <string.h>
 #include <signal.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <dbus/dbus-glib.h>
 #include <synce.h>
 #include <libnotify/notify.h>
-#include <glade/glade.h>
 
 #include "synce-trayicon.h"
 #include "gtop_stuff.h"
@@ -64,6 +63,7 @@ struct _SynceTrayIconPrivate {
   WmDeviceManager *device_list;
   GtkWidget *menu;
   NotifyNotification *notification;
+  gboolean show_disconnected;
 
   gboolean disposed;
 };
@@ -84,8 +84,13 @@ is_connected(SynceTrayIcon *self)
   return FALSE;
 }
 
+#if GTK_CHECK_VERSION(2,16,0)
+static void
+set_status_tooltip(SynceTrayIcon *self, GtkTooltip *tooltip)
+#else
 static void
 set_status_tooltips(SynceTrayIcon *self)
+#endif
 {
   SynceTrayIconPrivate *priv = SYNCE_TRAYICON_GET_PRIVATE (self);
 
@@ -99,7 +104,11 @@ set_status_tooltips(SynceTrayIcon *self)
   GList *device_names_iter = NULL;
 
   if (!(is_connected(self))) {
+#if GTK_CHECK_VERSION(2,16,0)
+          gtk_tooltip_set_text(tooltip, _("Not connected"));
+#else
           gtk_status_icon_set_tooltip(GTK_STATUS_ICON(self), _("Not connected"));
+#endif
           return;
   }
 
@@ -134,31 +143,49 @@ set_status_tooltips(SynceTrayIcon *self)
   }
 
   g_list_free(device_names);
+#if GTK_CHECK_VERSION(2,16,0)
+  gtk_tooltip_set_text(tooltip, tooltip_str);
+#else
   gtk_status_icon_set_tooltip(GTK_STATUS_ICON(self), tooltip_str);
+#endif
   g_free(tooltip_str);
   return;
 }
 
+#if GTK_CHECK_VERSION(2,16,0)
+static gboolean
+query_tooltip_cb(GtkWidget *widget, gint x, gint y, gboolean keyboard_mode, GtkTooltip *tooltip, gpointer user_data)
+{
+  set_status_tooltip(SYNCE_TRAYICON(user_data), tooltip);
+
+  /* show the tooltip */
+  return TRUE;
+}
+#endif
+
 static void
 set_icon(SynceTrayIcon *self)
 {
-  if (is_connected(self))
-    gtk_status_icon_set_from_icon_name(GTK_STATUS_ICON(self), SYNCE_STOCK_CONNECTED);
+  SynceTrayIconPrivate *priv = SYNCE_TRAYICON_GET_PRIVATE (self);
+
+
+  if (wm_device_manager_device_all_count(priv->device_list) > 0)
+          gtk_status_icon_set_visible(GTK_STATUS_ICON(self), TRUE);
   else
-    gtk_status_icon_set_from_icon_name(GTK_STATUS_ICON(self), SYNCE_STOCK_DISCONNECTED);
-	
-  while (gtk_events_pending ())
-    gtk_main_iteration ();
+          gtk_status_icon_set_visible(GTK_STATUS_ICON(self), priv->show_disconnected);
+
+  if (is_connected(self))
+          gtk_status_icon_set_from_icon_name(GTK_STATUS_ICON(self), SYNCE_STOCK_CONNECTED);
+  else
+          gtk_status_icon_set_from_icon_name(GTK_STATUS_ICON(self), SYNCE_STOCK_DISCONNECTED);
 }
 
 static gboolean 
 update(gpointer data)
 {
-  SynceTrayIcon *self = SYNCE_TRAYICON(data);
-
-  set_icon(self);
-  set_status_tooltips(self);
-
+#if !GTK_CHECK_VERSION(2,16,0)
+  set_status_tooltips(SYNCE_TRAYICON(data));
+#endif
   /* prevent function from running again when
    set with g_idle_add */
   return FALSE;
@@ -173,11 +200,14 @@ event_notification(SynceTrayIcon *self, const gchar *summary, const gchar *body)
   if (priv->notification) {
     notify_notification_close(priv->notification, NULL);
     g_object_unref (priv->notification);
+    priv->notification = NULL;
   }
 
   if (gtk_status_icon_is_embedded(GTK_STATUS_ICON(self))) {
           priv->notification = notify_notification_new_with_status_icon (summary, body, NULL, GTK_STATUS_ICON(self));
           notify_notification_show (priv->notification, NULL);
+  } else {
+          g_debug("%s: not embedded", G_STRFUNC);
   }
 }
 
@@ -202,7 +232,7 @@ trayicon_supply_password(SynceTrayIcon *self)
 
         GList *locked_devices = NULL;
         GList *tmplist = NULL;
-        GladeXML *xml;
+        GtkBuilder *builder;
         GtkWidget *password_dialog, *password_dialog_entry, *password_dialog_cancel;
         GtkWidget *password_dialog_ok, *password_dialog_pdaname, *password_dialog_save_pw;
         gchar *password = NULL;
@@ -223,15 +253,36 @@ trayicon_supply_password(SynceTrayIcon *self)
         if (!locked_devices)
                 return;
 
-        xml = glade_xml_new (SYNCE_DATA "synce_trayicon_properties.glade", "password_dialog", NULL);
+        builder = gtk_builder_new();
+        guint builder_res;
+        GError *error = NULL;
+#if GTK_CHECK_VERSION(2,14,0)
+        gchar *namelist[] = { "password_dialog", NULL };
 
-        password_dialog = glade_xml_get_widget (xml, "password_dialog");
+        builder_res = gtk_builder_add_objects_from_file(builder,
+                                                        SYNCE_DATA "synce_trayicon_properties.glade",
+                                                        namelist,
+                                                        &error);
+#else
+        builder_res = gtk_builder_add_from_file(builder,
+                                                SYNCE_DATA "synce_trayicon_properties.glade",
+                                                &error);
+#endif
+        if (builder_res == 0) {
+                g_critical("%s: failed to load interface file: %s", G_STRFUNC, error->message);
+                g_error_free(error);
+                error = NULL;
+        }
 
-        password_dialog_pdaname = glade_xml_get_widget (xml, "password_dialog_pdaname");
-        password_dialog_entry = glade_xml_get_widget (xml, "password_dialog_entry");
-        password_dialog_ok = glade_xml_get_widget (xml, "password_dialog_ok");
-        password_dialog_cancel = glade_xml_get_widget (xml, "password_dialog_cancel");
-        password_dialog_save_pw = glade_xml_get_widget (xml, "password_dialog_save_password");
+        password_dialog = GTK_WIDGET(gtk_builder_get_object(builder, "password_dialog"));
+
+        password_dialog_pdaname = GTK_WIDGET(gtk_builder_get_object(builder, "password_dialog_pdaname"));
+        password_dialog_entry = GTK_WIDGET(gtk_builder_get_object(builder, "password_dialog_entry"));
+        password_dialog_ok = GTK_WIDGET(gtk_builder_get_object(builder, "password_dialog_ok"));
+        password_dialog_cancel = GTK_WIDGET(gtk_builder_get_object(builder, "password_dialog_cancel"));
+        password_dialog_save_pw = GTK_WIDGET(gtk_builder_get_object(builder, "password_dialog_save_password"));
+
+        g_object_unref(builder);
 
         store = gtk_list_store_new (1, G_TYPE_STRING);
 
@@ -373,11 +424,11 @@ device_connected_cb(DccmClient *comms_client, gchar *pdaname, gpointer info, gpo
   g_debug("%s: looking for preexisting device %s", G_STRFUNC, pdaname);
   if ((device = wm_device_manager_find_by_name(priv->device_list, pdaname))) {
     g_debug("%s: Ignoring connection message for \"%s\", already connected", G_STRFUNC, pdaname);
-    g_object_unref(device);
     return;
   }
 
   wm_device_manager_add(priv->device_list, new_device);
+  set_icon(self);
 }
 
 static void
@@ -522,16 +573,18 @@ uninit_client_comms(SynceTrayIcon *self)
 static gboolean
 start_dccm (SynceTrayIcon *self)
 {
-  gchar *argv[3] = {
-    DCCM_BIN,""
+  GError *error = NULL;
+
+  gchar *argv[1] = {
+          DCCM_BIN
   };
-  gint argc = 1;
 
   if (!(dccm_is_running())) {
     g_debug("%s: starting %s", G_STRFUNC, DCCM_BIN);
-    if (gnome_execute_async(NULL, argc, argv) == -1) {
+    if (!g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error)) {
             event_notification(self, "Service failure", _("Can't start vdccm which is needed to communicate with the PDA. Make sure it is installed and try again."));
-            g_warning("%s: Failed to start %s", G_STRFUNC, DCCM_BIN);
+            g_warning("%s: Failed to start %s: %s", G_STRFUNC, DCCM_BIN, error->message);
+            g_error_free(error);
             return FALSE;
     }
   } else {
@@ -705,6 +758,9 @@ device_added_cb(GObject *obj, gchar *name, gpointer user_data)
   if (!new_device)
           return;
 
+  set_icon(self);
+  g_idle_add(update, self);
+
   module_run_connect(name);
 
   g_object_get(new_device, "hardware", &model, NULL);
@@ -716,8 +772,6 @@ device_added_cb(GObject *obj, gchar *name, gpointer user_data)
   g_free(model);
   g_free(platform);
   g_free(notify_string);
-
-  g_idle_add(update, self);
 }
 
 static void
@@ -731,9 +785,9 @@ device_removed_cb(GObject *obj, gchar *name, gpointer user_data)
 
   notify_string = g_strdup_printf("'%s' just disconnected.", name);
   event_notification(self, "PDA disconnected", notify_string);
-
   g_free(notify_string);
 
+  set_icon(self);
   g_idle_add(update, self);
 }
 
@@ -745,18 +799,23 @@ menu_explore (GtkWidget *menu_item, SynceTrayIcon *self)
 {
   const gchar *name = NULL;
   gchar *arg_str = NULL;
+  GError *error = NULL;
 
   GtkWidget *device_menu = gtk_widget_get_parent(menu_item);
   name = gtk_menu_get_title(GTK_MENU(device_menu));
 
   arg_str = g_strdup_printf("synce://%s/", name);
-
-  char *argv[2] = {
-    "nautilus", arg_str
+  
+  char *argv[4] = {
+          "nautilus", "-n", arg_str, NULL
   };
-  if (gnome_execute_async(NULL, 2, argv) == -1) {
-          g_warning("%s: failed to explore '%s'", G_STRFUNC, arg_str);
+  
+  if (!g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error)) {
+          g_warning("%s: failed to explore '%s': %s", G_STRFUNC, arg_str, error->message);
+          g_error_free(error);
+          return;
   }
+
   g_free(arg_str);
 }
 
@@ -804,13 +863,12 @@ menu_about (GtkWidget *button, SynceTrayIcon *icon)
     NULL
   };
 
-  GnomeProgram *prog_info = gnome_program_get ();
   about = gtk_about_dialog_new();
 
-  gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(about), gnome_program_get_human_readable_name (prog_info));
-  gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), gnome_program_get_app_version (prog_info));
+  gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(about), g_get_application_name());
+  gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), VERSION);
   gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about), _("Copyright (c) 2002, David Eriksson\n"
-							    "Copyright (c) 2007, Mark Ellis"));
+							    "Copyright (c) 2007 - 2009, Mark Ellis"));
   gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about), _("Displays information about devices connected through SynCE"));
   gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about), "http://www.synce.org");
   gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(about), authors);
@@ -986,6 +1044,7 @@ prefs_changed_cb (GConfClient *client, guint id,
 		  GConfEntry *entry, gpointer data)
 {
   SynceTrayIcon *self = SYNCE_TRAYICON(data);
+  SynceTrayIconPrivate *priv = SYNCE_TRAYICON_GET_PRIVATE (self);
 
   const gchar *key;
   const GConfValue *value;
@@ -1004,6 +1063,14 @@ prefs_changed_cb (GConfClient *client, guint id,
     return;
   }
 
+  if (!(g_ascii_strcasecmp(key, "/apps/synce/trayicon/show_disconnected"))) {
+    priv->show_disconnected = gconf_value_get_bool(value);
+    set_icon(self);
+
+    return;
+  }
+
+  return;
 }
 
 
@@ -1094,6 +1161,12 @@ synce_trayicon_init(SynceTrayIcon *self)
   g_signal_connect(G_OBJECT(self), "activate", G_CALLBACK(trayicon_activate_cb), self);
   g_signal_connect(G_OBJECT(self), "popup-menu", G_CALLBACK(trayicon_popup_menu_cb), self);
 
+#if GTK_CHECK_VERSION(2,16,0)
+  /* tooltip */
+  gtk_status_icon_set_has_tooltip(GTK_STATUS_ICON(self), TRUE);
+  g_signal_connect(G_OBJECT(self), "query-tooltip", G_CALLBACK(query_tooltip_cb), self);
+#endif
+
   priv->conf_watch_id = gconf_client_notify_add (priv->conf_client, 
                                                  "/apps/synce/trayicon", 
                                                  prefs_changed_cb, self, NULL, &error);
@@ -1103,10 +1176,18 @@ synce_trayicon_init(SynceTrayIcon *self)
           error = NULL;
   }                  
 
+  priv->show_disconnected = gconf_client_get_bool(priv->conf_client, "/apps/synce/trayicon/show_disconnected", &error);
+  if (error) {
+          g_critical("%s: Error getting '/apps/synce/trayicon/show_disconnected' from gconf: %s", G_STRFUNC, error->message);
+          g_error_free(error);
+          error = NULL;
+  }
+
   /* module initialisation */
   module_load_all();
 
   /* set initial state */
+  set_icon(self);
   update(self);
 
   /* dccm comms */
